Package: blase
Title: Bulk Linking Analysis for Single-cell Experiments
Version: 1.2.0
Authors@R: c(
  person("Andrew", "McCluskey", , "2117532m@student.gla.ac.uk", role = c("aut", "cre"), comment = c(ORCID = "0009-0004-4187-799X")),
  person("Toby", "Kettlewell", , , role = c("aut"), comment = c(ORCID = "0009-0001-1225-3318")),
  person("Adrian M.", "Smith", , , role = c("aut"), comment = c(ORCID="0000-0001-8833-2330")),
  person("Rhiannon", "Kundu", , , role = c("aut"), comment = c(ORCID="0000-0003-3970-5860")),
  person("David A.", "Gunn", , , role = c("aut"), comment = c(ORCID="0000-0001-9866-3221")),
  person("Thomas D.", "Otto", , , role=c("aut","ths"), comment=c(ORCID = "0000-0002-1246-7404")))
Description: BLASE is a method for finding where bulk RNA-seq data lies
        on a single-cell pseudotime trajectory. It uses a fast and
        understandable approach based on Spearman correlation, with
        bootstrapping to provide confidence. BLASE can be used to
        "date" bulk RNA-seq data, annotate cell types in scRNA-seq, and
        help correct for developmental phenotype differences in bulk
        RNA-seq experiments.
License: GPL (>= 3)
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
biocViews: Transcriptomics, SingleCell, Sequencing, GeneExpression,
        Transcription, RNASeq, TimeCourse, CellBiology, Software,
        CellBasedAssays
Imports: SummarizedExperiment, SingleCellExperiment, ggplot2, viridis,
        patchwork, Matrix, scater, methods, rlang, BiocParallel, boot,
        dplyr, mgcv, stats, MatrixGenerics, Seurat (>= 4.0.0), lsa
Suggests: knitr, rmarkdown, testthat (>= 3.2.3), covr, tradeSeq, scran,
        slingshot, tools, ami, reshape2, plyr, fs, sparseMatrixStats,
        ggVennDiagram, uwot, BiocStyle, DelayedMatrixStats, limma
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://andrewmccluskey-uog.github.io/blase/
BugReports: https://andrewmccluskey-uog.github.io/blase/issues
Depends: R (>= 4.5.0)
LazyData: false
Config/pak/sysreqs: libcairo2-dev cmake libfontconfig1-dev
        libfreetype6-dev libfribidi-dev libglpk-dev make
        libharfbuzz-dev libicu-dev libjpeg-dev libpng-dev libtiff-dev
        libuv1-dev libwebp-dev libxml2-dev libssl-dev python3
        zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 13:05:33 UTC
RemoteUrl: https://github.com/bioc/blase
RemoteRef: RELEASE_3_23
RemoteSha: 0176565467efd8121dc300999e79aa5f6f0f32b3
NeedsCompilation: no
Packaged: 2026-04-29 18:42:03 UTC; root
Author: Andrew McCluskey [aut, cre] (ORCID:
    <https://orcid.org/0009-0004-4187-799X>),
  Toby Kettlewell [aut] (ORCID: <https://orcid.org/0009-0001-1225-3318>),
  Adrian M. Smith [aut] (ORCID: <https://orcid.org/0000-0001-8833-2330>),
  Rhiannon Kundu [aut] (ORCID: <https://orcid.org/0000-0003-3970-5860>),
  David A. Gunn [aut] (ORCID: <https://orcid.org/0000-0001-9866-3221>),
  Thomas D. Otto [aut, ths] (ORCID:
    <https://orcid.org/0000-0002-1246-7404>)
Maintainer: Andrew McCluskey <2117532m@student.gla.ac.uk>
Built: R 4.6.0; ; 2026-04-29 20:25:05 UTC; unix
