# Generated by roxygen2: do not edit by hand

export("bulk_name<-")
export("genes<-")
export(BlaseData)
export(annotate_sce)
export(as.BlaseData)
export(assign_pseudotime_bins)
export(best_bin)
export(best_correlation)
export(bootstrap_iterations)
export(bulk_name)
export(calculate_gene_peakedness)
export(evaluate_parameters)
export(evaluate_top_n_genes)
export(find_best_params)
export(gene_peakedness_spread_selection)
export(genes)
export(get_bins_as_bulk)
export(get_top_n_genes)
export(map_all_best_bins)
export(map_best_bin)
export(mapping_history)
export(metric)
export(plot_bin_population)
export(plot_find_best_params_results)
export(plot_gene_peakedness)
export(plot_mapping_result)
export(plot_mapping_result_corr)
export(plot_mapping_result_heatmap)
export(smooth_gene)
export(strong_mapping)
export(top_2_distance)
exportClasses(BlaseData)
exportClasses(MappingResult)
exportMethods(as.BlaseData)
exportMethods(assign_pseudotime_bins)
exportMethods(plot_bin_population)
exportMethods(plot_mapping_result)
exportMethods(show)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(Matrix,rowSums)
importFrom(MatrixGenerics,rowMaxs)
importFrom(Seurat,as.SingleCellExperiment)
importFrom(SingleCellExperiment,counts)
importFrom(SummarizedExperiment,colData)
importFrom(boot,boot)
importFrom(dplyr,bind_rows)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_add)
importFrom(ggplot2,labs)
importFrom(ggplot2,sym)
importFrom(ggplot2,theme)
importFrom(lsa,cosine)
importFrom(methods,new)
importFrom(methods,show)
importFrom(methods,validObject)
importFrom(mgcv,gam)
importFrom(mgcv,predict.gam)
importFrom(patchwork,plot_annotation)
importFrom(patchwork,plot_layout)
importFrom(patchwork,wrap_plots)
importFrom(rlang,check_installed)
importFrom(scater,plotUMAP)
importFrom(stats,dist)
importFrom(stats,runif)
importFrom(viridis,scale_color_viridis)
