% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{MCA_PF_SCE}
\alias{MCA_PF_SCE}
\title{Malaria Cell Atlas Plasmodium falciparum for BLASE Vignette}
\format{
An object of class \code{SingleCellExperiment} with 1746 rows and 2500 columns.
}
\source{
\url{https://www.malariacellatlas.org/atlas/plasmodium-falciparum-atlas/}
}
\usage{
MCA_PF_SCE
}
\description{
Data from the Malaria Cell Atlas, with the following additional processing:
\enumerate{
\item Genes renamed to match bulk samples in vignette
\item Subset to 2500 cells
\item Normalised
\item Highly variable genes identified
\item Pseudotime calculated
\item Genes subset to include a spread of those found to have
high ratios by BLASE's "Gene Peakedness" measure.
}
}
\concept{data}
\keyword{datasets}
