% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tradeSeq_BLASE_example_sce}
\alias{tradeSeq_BLASE_example_sce}
\title{TradeSeq Example SCE for BLASE Vignette}
\format{
An object of class \code{SingleCellExperiment} with 240 rows and 1565 columns.
}
\source{
\url{https://bioconductor.org/packages/devel/bioc/vignettes/tradeSeq/inst/doc/tradeSeq.html}
}
\usage{
tradeSeq_BLASE_example_sce
}
\description{
Data from the TradeSeq vignette, with the following additional
processing applied:
}
\details{
\enumerate{
\item Pseudotime calculated
\item TradeSeq applied
\item Log normalised and normalised counts calculated
\item Erythrocyte cell type removed
\item UMAP calculated
}
}
\concept{data}
\keyword{datasets}
