% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blase_data_class.R
\name{pseudobulk_bins<-}
\alias{pseudobulk_bins<-}
\alias{pseudobulk_bins<-,BlaseData-method}
\title{Set genes of a BLASE Data object.}
\usage{
pseudobulk_bins(x) <- value

\S4method{pseudobulk_bins}{BlaseData}(x) <- value
}
\arguments{
\item{x}{a \link{BlaseData} object}

\item{value}{List of dataframes. Each dataframe is the normalised counts of
cells by genes in each pseudotime bin. List index is the pseudotime bin.}
}
\value{
Nothing
}
\description{
Set genes of a BLASE Data object.
}
\concept{blase-object}
\keyword{internal}
