% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{painter_microarray}
\alias{painter_microarray}
\title{Painter 2018 Plasmodium falciparum 48h asexual lifecycle microarray data}
\format{
An object of class \code{data.frame} with 1731 rows and 48 columns.
}
\source{
\url{https://zenodo.org/records/16615703}
}
\usage{
painter_microarray
}
\description{
Data originally from \url{https://doi.org/10.1038/s41467-018-04966-3}. Used as
generated in the BLASE reproducibility documents available at
\url{https://zenodo.org/records/16615703}, however genes have been subset to
reduce file size.
}
\concept{data}
\keyword{datasets}
