% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mapping.R
\name{PRIVATE_.ci}
\alias{PRIVATE_.ci}
\title{.ci}
\usage{
PRIVATE_.ci(x, conf.level = 0.95)
}
\arguments{
\item{x}{data to calculate ci for}

\item{conf.level}{confidence level to calculate}
}
\value{
confidence interval results
}
\description{
Originally implemented in RVAidemoire
Version 0.9-83-7.
}
\keyword{internal}
