% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blase_data_class.R
\name{bins<-}
\alias{bins<-}
\alias{bins<-,BlaseData-method}
\title{Set genes of a BLASE Data object.}
\usage{
bins(x) <- value

\S4method{bins}{BlaseData}(x) <- value
}
\arguments{
\item{x}{a \link{BlaseData} object}

\item{value}{Vector of integers The new value for bins slot}
}
\value{
Nothing
}
\description{
Set genes of a BLASE Data object.
}
\concept{blase-object}
\keyword{internal}
