% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mapping.R
\name{PRIVATE_spearman.ci}
\alias{PRIVATE_spearman.ci}
\title{Confidence interval of a Spearman's rank correlation coefficient}
\usage{
PRIVATE_spearman.ci(var1, var2, nrep = 1000, conf.level = 0.95)
}
\arguments{
\item{var1}{numeric vector (first variable).}

\item{var2}{nuermic verctor (second variable).}

\item{nrep}{number of replicates for bootstrapping.}

\item{conf.level}{confidence level of the interval.}
}
\value{
description method name of the test.

data.name a character string giving the name(s) of the data.

conf.level confidence level.

rep number of replicates.

estimate Spearman's rank correlation coefficient.

conf.int confidence interval.
}
\description{
Computes the confidence interval of a Spearman's rank correlation
coefficient by bootstraping. Originally implemented in RVAidemoire
Version 0.9-83-7.
}
\keyword{internal}
